"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _database() {
  const data = require("@nocobase/database");

  _database = function _database() {
    return data;
  };

  return data;
}

var _triggers = _interopRequireDefault(require("../triggers"));

var _constants = require("../constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

class WorkflowModel extends _database().Model {
  static mount() {
    var _this = this;

    return _asyncToGenerator(function* () {
      const collection = _this.database.getCollection('workflows');

      const workflows = yield collection.repository.find({
        filter: {
          enabled: true
        }
      });
      workflows.forEach(workflow => {
        workflow.toggle();
      });

      _this.addHook('afterCreate', model => model.toggle());

      _this.addHook('afterUpdate', model => model.toggle());

      _this.addHook('afterDestroy', model => model.toggle(false));
    })();
  }

  getHookId() {
    return `workflow-${this.get('id')}`;
  }

  getTransaction(options) {
    if (!this.useTransaction) {
      return undefined;
    }

    return options.transaction && !options.transaction.finished ? options.transaction : this.constructor.database.sequelize.transaction();
  }

  toggle(enable) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const type = _this2.get('type');

      const _triggers$get = _triggers.default.get(type),
            on = _triggers$get.on,
            off = _triggers$get.off;

      if (typeof enable !== 'undefined' ? enable : _this2.get('enabled')) {
        on.call(_this2, _this2.trigger.bind(_this2));
      } else {
        off.call(_this2);
      }
    })();
  }

  trigger(context, options) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      // `null` means not to trigger
      if (context === null) {
        return;
      }

      const transaction = yield _this3.getTransaction(options);

      if (_this3.useTransaction) {
        const existed = yield _this3.countExecutions({
          where: {
            transaction: transaction.id
          },
          transaction
        });

        if (existed) {
          console.warn(`workflow ${_this3.id} has already been triggered in same execution (${transaction.id}), and newly triggering will be skipped.`);
          return;
        }
      }

      const execution = yield _this3.createExecution({
        context,
        status: _constants.EXECUTION_STATUS.STARTED,
        useTransaction: _this3.useTransaction,
        transaction: transaction.id
      }, {
        transaction
      });
      execution.workflow = _this3;
      yield execution.start({
        transaction
      });

      if (!_this3.executed) {
        yield _this3.update({
          executed: true
        }, {
          transaction
        });
      }

      if (transaction && (!options.transaction || options.transaction.finished)) {
        yield transaction.commit();
      }

      return execution;
    })();
  }

}

exports.default = WorkflowModel;